@@ -1,7 +1,7 @@
 /*
  * defines used for messaging
  *
- * Copyright (c) 2001        Nico Schottelius <nico@schottelius.org>
+ * Copyright (c) 2001,2002    Nico Schottelius <nico@schottelius.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -41,11 +41,11 @@
 #define GPM_PR_OOPS   FL,GPM_STAT_OOPS
 
 /* messages */
-#define GPM_TEXT_INFO   "info: "
-#define GPM_TEXT_WARN   "warning: "
-#define GPM_TEXT_DEBUG  "*** debug ***: "
-#define GPM_TEXT_ERR    "=== ERROR ===: "
-#define GPM_TEXT_OOPS   "oops(): "
+#define GPM_TEXT_INFO   "*** info "
+#define GPM_TEXT_WARN   "*** warning "
+#define GPM_TEXT_DEBUG  "*** debug "
+#define GPM_TEXT_ERR    "*** err "
+#define GPM_TEXT_OOPS   "O0o.oops(): "
 
 /* strings */
 #define GPM_STRING_INFO   GPM_TEXT_INFO FLP
@@ -62,6 +62,7 @@
 /* messages */
 
 /* info */
+#define GPM_MESS_VERSION            "gpm " GPM_RELEASE " " GPM_RELEASE_DATE
 #define GPM_MESS_STARTED            "Started gpm successfully. Entered daemon mode."
 #define GPM_MESS_KILLED             "Killed gpm(%d)."
 #define GPM_MESS_SKIP_DATA          "Skipping a data packet (?)"
@@ -82,7 +83,7 @@
          "    -k               kill a running gpm, to start X with a busmouse\n" \
          "    -l charset       loads the inword() LUT (default '%s')\n" \
          "    -m mouse-device  sets mouse device\n" \
-         "    -M               enable multiple mouse. Following options refer to\n" \
+         "    -M               enable multiple mice. Following options refer to\n" \
          "                     the second device. Forces \"-R\"\n" \
          "    -o options       decoder-specific options (e.g.: \"dtr\", \"rts\")\n" \
          "    -p               draw the pointer while striking a selection\n" \
@@ -97,7 +98,14 @@
          "                     Use a non-existent type (e.g. \"help\") to get a list\n" \
          "    -T               test: read mouse, no clients\n" \
          "    -v               print version and exit\n" \
-         "    -V verbosity     increase number of logged messages\n"
+         "    -V verbosity     increase number of logged messages\n\n\n" \
+         "    Examples:\n\n" \
+         "    gpm -m /dev/misc/psaux -t ps2    to start with a ps2 mouse\n" \
+         "    gpm -m /dev/tts/0 -t mman        to use mouse man on COM1\n\n"
+
+/* Later ... but first when it's running */
+/*         "    -u               autodetect mice [not implemented yet]\n" \ */
+
 #define GPM_MESS_IMPS2_AUTO         "imps2: Auto-detected intellimouse PS/2"
 #define GPM_MESS_IMPS2_PS2          "imps2: Auto-detected standard PS/2"
 #define GPM_MESS_AVAIL_MYT          "Available mouse types are:\n\n" \
@@ -108,7 +116,6 @@
 #define GPM_MESS_DOUBLE_S           "%s: %s"
 #define GPM_MESS_DOUBLE_I           "%i - %i:"
 #define GPM_MESS_X_Y_VAL            "x %i, y %i"
-#define GPM_MESS_S_S_N              "%s %s, %s\n"
 
 
 /* mem */
@@ -125,6 +132,7 @@
 #define GPM_MESS_NOTWRITE           "Can you write to %s?"
 #define GPM_MESS_WRITE_ERR          "write(): %s"
 #define GPM_MESS_OPEN_CON           "Opening console failed."
+#define GPM_MESS_OPEN_SERIALCON     "We seem to be on a serial console."
 #define GPM_MESS_READ_FIRST         "Error in read()ing first: %s"
 #define GPM_MESS_READ_REST          "Error in read()ing rest: %s"
 #define GPM_MESS_REMOVE_FILES       "Removing files %s and %s"
@@ -152,8 +160,7 @@
 #define GPM_MESS_SOCKET_OLD         "Socket too old in processConn"
 #define GPM_MESS_GETSOCKOPT         "getsockopt(SO_PEERCRED): %s"
 #define GPM_MESS_STAT_FAILS         "stat on %s fails in processConn"
-#define GPM_MESS_NEED_MDEV          "Since gpm-1.20.0 you need to specify the" \
-                                    "mouse device. gpm -m device is your friend"
+#define GPM_MESS_NEED_MDEV          "Please use -m /dev/mouse -t protocol"
 #define GPM_MESS_MOUSE_INIT         "mouse initialization failed"
 #define GPM_MESS_SOCKET_PROB        "socket()"
 #define GPM_MESS_BIND_PROB          "Problem binding %s"
@@ -185,7 +192,8 @@
 #define GPM_MESS_UNKNOWN_MOD_1      "%s: %s :%i: Unknown modifier \"%s\""
 #define GPM_MESS_INCORRECT_COORDS   "%s: %s :%i: Incorrect chord \"%s\""
 #define GPM_MESS_INCORRECT_LINE     "%s: %s :%i: Incorrect line:\"%s\""
-
+#define GPM_MESS_FIRST_DEV          "Use -m device -t protocol [-o options]!"
+#define GPM_MESS_ELO_CALIBRATE      "%s: etouch: calibration file %s absent or invalid, using defaults"
 
 /* warnings */
 #define GPM_MESS_REQUEST_ON         "Request on vc %i > %i"
