@@ -4,7 +4,7 @@
  * Copyright (C) 1993        Andrew Haylett <ajh@gec-mrc.co.uk>
  * Copyright (C) 1994-2000   Alessandro Rubini <rubini@linux.it>
  * Copyright (C) 1998,1999   Ian Zimmerman <itz@rahul.net>
- * Copyright (C) 2001,2002   Nico Schottelius <nicos@pcsystems.de>
+ * Copyright (C) 2001-2008   Nico Schottelius <nico-gpm2008 at schottelius.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -72,10 +72,12 @@
 
 
 #include "headers/gpmInt.h"
+#include "headers/daemon.h"
 #include "headers/twiddler.h"
 #include "headers/synaptics.h"
 #include "headers/message.h"
 
+
 /*========================================================================*/
 /* Parsing argv: helper dats struct function (should they get elsewhere?) */
 /*========================================================================*/
@@ -101,7 +103,7 @@ typedef struct argv_helper {
 
 static int parse_argv(argv_helper *info, int argc, char **argv)
 {
-   int i, j, errors = 0;
+   int i, j = 0, errors = 0;
    long l;
    argv_helper *p;
    char *s, *t;
@@ -119,7 +121,7 @@ static int parse_argv(argv_helper *info, int argc, char **argv)
       }
       if (p->type == ARGV_END) { /* not found */
          fprintf(stderr, "%s: Uknown option \"%s\" for pointer \"%s\"\n",
-                  prgname, argv[i], argv[0]);
+                  option.progname, argv[i], argv[0]);
          errors++;
          continue;
       }
@@ -133,7 +135,7 @@ static int parse_argv(argv_helper *info, int argc, char **argv)
       switch(p->type) {
          case ARGV_BOOL:
             if (*s) {
-               gpm_report(GPM_PR_ERR,GPM_MESS_OPTION_NO_ARG,prgname,p->name,s);
+               gpm_report(GPM_PR_ERR,GPM_MESS_OPTION_NO_ARG, option.progname,p->name,s);
                errors++;
             }
             *(p->u.iptr) = p->value;
@@ -144,7 +146,7 @@ static int parse_argv(argv_helper *info, int argc, char **argv)
          case ARGV_INT:
             l = strtol(s, &t, base);
             if (*t) {
-               gpm_report(GPM_PR_ERR,GPM_MESS_INVALID_ARG, prgname, s, p->name);
+               gpm_report(GPM_PR_ERR,GPM_MESS_INVALID_ARG, option.progname, s, p->name);
                errors++;
                break;
             }
@@ -159,7 +161,7 @@ static int parse_argv(argv_helper *info, int argc, char **argv)
             break;
       }
    } /* for i in argc */
-   if (errors) gpm_report(GPM_PR_ERR,GPM_MESS_CONT_WITH_ERR, prgname);
+   if (errors) gpm_report(GPM_PR_ERR,GPM_MESS_CONT_WITH_ERR, option.progname);
    return errors;
 }
 
@@ -190,8 +192,8 @@ static int option_modem_lines(int fd, int argc, char **argv)
 
    if (argc<2) return 0;
    if (argc > 2) {
-      gpm_report(GPM_PR_ERR,GPM_MESS_TOO_MANY_OPTS,prgname, argv[0]);
-      errno = EINVAL; /* used by gpm_gpm_oops(), if the caller reports failure */
+      gpm_report(GPM_PR_ERR,GPM_MESS_TOO_MANY_OPTS,option.progname, argv[0]);
+      errno = EINVAL; /* used by gpm_oops(), if the caller reports failure */
       return -1;
    }
    err = parse_argv(optioninfo, argc, argv);
@@ -421,16 +423,54 @@ static int M_geni(Gpm_Event *state,  unsigned char *data)
 /* m$ 'Intellimouse' (steveb 20/7/97) */
 static int M_ms3(Gpm_Event *state,  unsigned char *data)
 {
+   state->wdx = state->wdy = 0;
    state->buttons= ((data[0] & 0x20) >> 3)   /* left */
       | ((data[3] & 0x10) >> 3)   /* middle */
-      | ((data[0] & 0x10) >> 4);   /* right */
-   state->dx=      (signed char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
-   state->dy=      (signed char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
-   /* wheel (dz??) is (data[3] & 0x0f) */
+      | ((data[0] & 0x10) >> 4)   /* right */
+      | (((data[3] & 0x0f) == 0x0f) * GPM_B_UP)     /* wheel up */
+      | (((data[3] & 0x0f) == 0x01) * GPM_B_DOWN);  /* wheel down */
+   state->dx = (signed char) (((data[0] & 0x03) << 6) | (data[1] & 0x3F));
+   state->dy = (signed char) (((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
+
+   switch (data[3] & 0x0f) {
+      case 0x0e: state->wdx = +1; break;
+      case 0x02: state->wdx = -1; break;
+      case 0x0f: state->wdy = +1; break;
+      case 0x01: state->wdy = -1; break;
+   }
 
    return 0;
 }
 
+static int R_ms3(Gpm_Event *state, int fd)
+{
+
+   int dx, dy;
+   char buf[4] = {0, 0, 0, 0};
+   
+   buf[0] |= 0x40;
+   
+   if (state->buttons & GPM_B_LEFT)     buf[0] |= 0x20;
+   if (state->buttons & GPM_B_MIDDLE)   buf[3] |= 0x10;
+   if (state->buttons & GPM_B_RIGHT)    buf[0] |= 0x10;
+   if (state->buttons & GPM_B_UP)       buf[3] |= 0x0f;
+   if (state->buttons & GPM_B_DOWN)     buf[3] |= 0x01;
+
+   dx = limit_delta(state->dx, -128, 127);
+   buf[1]   = dx & ~0xC0;
+   buf[0]  |= (dx & 0xC0) >> 6;
+
+   dy = limit_delta(state->dy, -128, 127);
+   buf[2] = dy & ~0xC0;
+   buf[0] |= (dy & 0xC0) >> 4;
+
+   /* wheel */
+   if       (state->wdy > 0)   buf[3] |= 0x0f;
+   else if  (state->wdy < 0)   buf[3] |= 0x01;
+   
+   return write(fd,buf,4);
+}
+
 /* M_brw is a variant of m$ 'Intellimouse' the middle button is different */
 static int M_brw(Gpm_Event *state,  unsigned char *data)
 {
@@ -470,6 +510,16 @@ static int M_sun(Gpm_Event *state,  unsigned char *data)
    return 0;
 }
 
+static int R_sun(Gpm_Event *state, int fd)
+{
+  signed char buffer[3];
+
+  buffer[0]= (state->buttons ^ 0x07) | 0x80;
+  buffer[1]= state->dx;
+  buffer[2]= -(state->dy);
+  return write(fd,buffer,3);
+}
+
 static int M_msc(Gpm_Event *state,  unsigned char *data)
 {
    state->buttons= (~data[0]) & 0x07;
@@ -512,9 +562,10 @@ static int R_imps2(Gpm_Event *state, int fd)
       (dy > 0 ? 0x20 : 0);
    buffer[1] = dx & 0xFF;
    buffer[2] = (-dy) & 0xFF;
-   buffer[3] = 
-      (state->buttons & GPM_B_UP ? -1 : 0) + 
-      (state->buttons & GPM_B_DOWN ? 1 : 0);
+   if (state->wdy > 0) buffer[3] = 0xff;
+   if (state->wdy < 0) buffer[3] = 0x01;
+   if (state->wdx > 0) buffer[3] = 0xfe;
+   if (state->wdx < 0) buffer[3] = 0x02;
   
    return write(fd,buffer,4);
 
@@ -561,8 +612,8 @@ static int M_ps2(Gpm_Event *state,  unsigned char *data)
       !!(data[0]&2) * GPM_B_RIGHT +
       !!(data[0]&4) * GPM_B_MIDDLE;
 
-   if (data[0]==0 && opt_glidepoint_tap) /* by default this is false */
-      state->buttons = tap_active = opt_glidepoint_tap;
+   if (data[0]==0 && (which_mouse->opt_glidepoint_tap)) /* by default this is false */
+      state->buttons = tap_active = (which_mouse->opt_glidepoint_tap);
    else if (tap_active) {
       if (data[0]==8)
          state->buttons = tap_active = 0;
@@ -588,13 +639,50 @@ static int M_ps2(Gpm_Event *state,  unsigned char *data)
    return 0;
 }
 
-/* uses signed char *date, which should be unsigned; is really signed needed?*/
-static int M_imps2(Gpm_Event *state,  signed char *data)
+static int M_imps2(Gpm_Event *state,  unsigned char *data)
 {
-   int ret = M_ps2(state, data);
-   state->buttons |= (data[3]<0) * GPM_B_UP + (data[3]>0) * GPM_B_DOWN; 
-    
-   return ret;
+   
+   static int tap_active=0;         /* there exist glidepoint ps2 mice */
+   state->wdx  = state->wdy   = 0;  /* Clear them.. */
+   state->dx   = state->dy    = state->wdx = state->wdy = 0;
+
+   state->buttons= ((data[0] & 1) << 2)   /* left              */
+      | ((data[0] & 6) >> 1);             /* middle and right  */
+   
+   if (data[0]==0 && (which_mouse->opt_glidepoint_tap)) // by default this is false
+      state->buttons = tap_active = (which_mouse->opt_glidepoint_tap);
+   else if (tap_active) {
+      if (data[0]==8)
+         state->buttons = tap_active = 0;
+      else state->buttons = tap_active;
+   }
+
+   /* Standard movement.. */
+   state->dx = (data[0] & 0x10) ? data[1] - 256 : data[1];
+   state->dy = (data[0] & 0x20) ? -(data[2] - 256) : -data[2];
+   
+   /* The wheels.. */
+   unsigned char wheel = data[3] & 0x0f;
+   if (wheel > 0) {
+     // use the event type GPM_MOVE rather than GPM_DOWN for wheel movement
+     // to avoid single/double/triple click processing:
+     switch (wheel) {
+       /* rodney 13/mar/2008
+        * The use of GPM_B_UP / GPM_B_DOWN is very unclear; 
+        *  only mouse type ms3 uses these
+        * For this mouse, we only support the relative movement
+        * i.e. no button is set (same as mouse movement), wdy changes +/-
+        *  according to wheel movement (+ for rolling away from user)
+        * wdx (horizontal scroll) is for a second wheel. They do exist! */
+        case 0x0f: state->wdy = +1; break;
+        case 0x01: state->wdy = -1; break;
+        case 0x0e: state->wdx = +1; break;
+        case 0x02: state->wdx = -1; break;
+     }
+   }
+
+   return 0;
+
 }
 
 static int M_netmouse(Gpm_Event *state,  unsigned char *data)
@@ -686,11 +774,12 @@ static int M_mman(Gpm_Event *state,  unsigned char *data)
     */
     static unsigned char prev=0;
     static Gpm_Type *mytype=mice; /* it is the first */
-    unsigned char b;
+    unsigned char b = (*data>>4);
 
    if(data[1]==GPM_EXTRA_MAGIC_1 && data[2]==GPM_EXTRA_MAGIC_2) {
       /* got unexpected fourth byte */
-      if ((b=(*data>>4)) > 0x3) return -1;  /* just a sanity check */
+      if (b > 0x3) return -1;  /* just a sanity check */
+      //if ((b=(*data>>4)) > 0x3) return -1;  /* just a sanity check */
       state->dx=state->dy=0;
     
       mytype->packetlen=4;
@@ -713,7 +802,7 @@ static int M_mman(Gpm_Event *state,  unsigned char *data)
          mytype->getextra=1;
       } else {
          if (b & 0x2) prev |= GPM_B_MIDDLE;
-         if (b & 0x1) prev |= opt_glidepoint_tap;
+         if (b & 0x1) prev |= (which_mouse->opt_glidepoint_tap);
       }
    }
    state->buttons=prev;
@@ -1174,7 +1263,7 @@ static int M_mtouch(Gpm_Event *state,  unsigned char *data)
 
    if (avgx < 0) { /* press event */
       GET_TIME(tv);
-      if (DIF_TIME(uptv, tv) < opt_time) {
+      if (DIF_TIME(uptv, tv) < (which_mouse->opt_time)) {
          /* count as button press placed at finger-up pixel */
          state->buttons = GPM_B_LEFT;
          realposx = avgx = upx; state->x = REAL_TO_XCELL(realposx);
@@ -1279,7 +1368,7 @@ static int M_gunze(Gpm_Event *state,  unsigned char *data)
       GET_TIME(tv);
       timediff = DIF_TIME(uptv, tv);
       released = 0;
-      if (timediff > gunze_debounce && timediff < opt_time) {
+      if (timediff > gunze_debounce && timediff < (which_mouse->opt_time)) {
          /* count as button press placed at finger-up pixel */
          dragging = 1;
          state->buttons = GPM_B_LEFT;
@@ -1320,6 +1409,143 @@ static int M_gunze(Gpm_Event *state,  unsigned char *data)
    #undef DIF_TIME
 }
 
+/*
+ * This decoder is copied and adapted from the above mtouch.
+ */
+static int elo_click_ontouch = 0; /* the bigger the smoother */
+static int M_etouch(Gpm_Event *state,  unsigned char *data)
+{ /*
+   * This is a simple decoder for the EloTouch touch screen devices. 
+   * ELO format SmartSet UTsXXYYZZc 9600,N,8,1
+   * c=checksum = 0xAA+'T'+'U'+s+X+X+Y+Y+Z+Z (XXmax=YYmax=0x0FFF=4095)
+   * s=status   bit 0=init touch  1=stream touch  2=release
+   */
+#define ELO_CLICK_ONTOUCH	/* ifdef then ButtonPress on first Touch 
+				         else first Move then Touch*/
+  int x, y;
+  static int avgx=-1, avgy;       /* average over time, for smooth feeling */
+  static int upx, upy;            /* keep track of last finger-up place */
+  static struct timeval uptv, tv; /* time of last up, and down events */
+
+  #define REAL_TO_XCELL(x) (x * win.ws_col / 0x3FFF)
+  #define REAL_TO_YCELL(y) (y * win.ws_row / 0x3FFF)
+
+  #define GET_TIME(tv) (gettimeofday(&tv, (struct timezone *)NULL))
+  #define DIF_TIME(t1,t2) ((t2.tv_sec -t1.tv_sec) *1000+ \
+                           (t2.tv_usec-t1.tv_usec)/1000)
+
+  if (data[2]&0x04)	/* FINGER UP - Release */
+  { upx = avgx;		/* ignore this x, y */
+    upy = avgy;		/* store Finger UP possition */
+    GET_TIME(uptv);	/* set time for the next finger-down */
+    tv.tv_sec = 0;	/* NO DRAG */
+    avgx=-1;		/* FINGER IS UP */
+    state->buttons = 0;
+    return 0;
+  }
+
+  /* NOW WE HAVe FINGER DOWN */
+  x = data[3] | (data[4]<<8); x&=0xfff;
+  y = data[5] | (data[6]<<8); x&=0xfff;
+  x = REALPOS_MAX * (x - gunze_calib[0])/(gunze_calib[2]-gunze_calib[0]);
+  y = REALPOS_MAX * (y - gunze_calib[1])/(gunze_calib[3]-gunze_calib[1]);
+  if (x<0) x = 0; if (x > REALPOS_MAX) x = REALPOS_MAX;
+  if (y<0) y = 0; if (y > REALPOS_MAX) y = REALPOS_MAX;
+
+  if (avgx < 0)		/* INITIAL TOUCH, FINGER WAS UP */
+  { GET_TIME(tv);
+    state->buttons = 0;
+    if (DIF_TIME(uptv, tv) < (which_mouse->opt_time))
+    {	/* if Initial Touch immediate after finger UP then start DRAG */
+	x=upx; y=upy;  /* A:start DRAG at finger-UP position */ 
+	if (elo_click_ontouch==0)	state->buttons = GPM_B_LEFT;
+    }
+    else /*  1:MOVE to Initial Touch position */
+    {	upx=x; upy=y; /* store position of Initial Touch into upx, upy */
+	if (elo_click_ontouch==0)	tv.tv_sec=0; /* no DRAG */
+    }
+    realposx = avgx = x; state->x = REAL_TO_XCELL(realposx);
+    realposy = avgy = y; state->y = REAL_TO_YCELL(realposy);
+    return 0;
+  } /* endof INITIAL TOUCH */
+
+
+  state->buttons = 0; /* Motion event */
+  if (tv.tv_sec)	/* draging or elo_click_ontouch */
+  { state->buttons = GPM_B_LEFT;
+    if (elo_click_ontouch) 
+    {	x=avgx=upx;	/* 2:BUTTON PRESS at Initial Touch position */
+	y=avgy=upy;
+	tv.tv_sec=0;   /* so next time 3:MOVE again until Finger UP*/
+    } /* else B:continue DRAG to current possition */
+  }
+
+  realposx = avgx = (9*avgx + x)/10; state->x = REAL_TO_XCELL(realposx);
+  realposy = avgy = (9*avgy + y)/10; state->y = REAL_TO_YCELL(realposy);
+  return 0;
+
+  #undef REAL_TO_XCELL
+  #undef REAL_TO_YCELL
+  #undef GET_TIME
+  #undef DIF_TIME
+}
+
+
+/* Support for DEC VSXXX-AA and VSXXX-GA serial mice used on   */
+/* DECstation 5000/xxx, DEC 3000 AXP and VAXstation 4000       */
+/* workstations                                                */
+/* written 2001/07/11 by Karsten Merker (merker@linuxtag.org)  */
+/* modified (completion of the protocol specification and      */
+/* corresponding correction of the protocol identification     */
+/* mask) 2001/07/12 by Maciej W. Rozycki (macro@ds2.pg.gda.pl) */
+
+static int M_vsxxx_aa(Gpm_Event *state, unsigned char *data)
+{
+
+/* The mouse protocol is as follows:
+ * 4800 bits per second, 8 data bits, 1 stop bit, odd parity
+ * 3 data bytes per data packet:
+ *              7     6     5     4     3     2     1     0
+ * First Byte:  1     0     0   SignX SignY  LMB   MMB   RMB
+ * Second Byte  0     DX    DX    DX    DX    DX    DX    DX
+ * Third Byte   0     DY    DY    DY    DY    DY    DY    DY
+ *
+ * SignX:     sign bit for X-movement
+ * SignY:     sign bit for Y-movement
+ * DX and DY: 7-bit-absolute values for delta-X and delta-Y, sign extensions
+ *            are in SignX resp. SignY.
+ *
+ * There are a few commands the mouse accepts:
+ * "D" selects the prompt mode,
+ * "P" requests a mouse's position (also selects the prompt mode),
+ * "R" selects the incremental stream mode,
+ * "T" performs a self test and identification (power-up-like),
+ * "Z" performs undocumented test functions (a byte follows).
+ * Parity as well as bit #7 of commands are ignored by the mouse.
+ *
+ * 4 data bytes per self test packet (useful for hot-plug):
+ *              7     6     5     4     3     2     1     0
+ * First Byte:  1     0     1     0     R3    R2    R1    R0
+ * Second Byte  0     M2    M1    M0    0     0     1     0
+ * Third Byte   0     E6    E5    E4    E3    E2    E1    E0
+ * Fourth Byte  0     0     0     0     0    LMB   MMB   RMB
+ *
+ * R3-R0:     revision,
+ * M2-M0:     manufacturer location code,
+ * E6-E0:     error code:
+ *            0x00-0x1f: no error (fourth byte is button state),
+ *            0x3d:      button error (fourth byte specifies which),
+ *            else:      other error.
+ * 
+ * The mouse powers up in the prompt mode but we use the stream mode.
+ */
+
+  state->buttons = data[0]&0x07;
+  state->dx = (data[0]&0x10) ? data[1] : -data[1];
+  state->dy = (data[0]&0x08) ? -data[2] : data[2];
+  return 0;
+}
+
 /*  Genius Wizardpad tablet  --  Matt Kimball (mkimball@xmission.com)  */
 static int wizardpad_width = -1;
 static int wizardpad_height = -1;
@@ -1444,7 +1670,7 @@ static Gpm_Type* I_serial(int fd, unsigned short flags,
 
    /* Non mman: change from any available speed to the chosen one */
    for (i=9600; i>=1200; i/=2)
-      setspeed(fd, i, opt_baud, (type->fun != M_mman) /* write */, flags);
+      setspeed(fd, i, (which_mouse->opt_baud), (type->fun != M_mman) /* write */, flags);
 
    /*
     * reset the MouseMan/TrackMan to use the 3/4 byte protocol
@@ -1454,7 +1680,7 @@ static Gpm_Type* I_serial(int fd, unsigned short flags,
    if (type->fun==M_mman) {
       setspeed(fd, 1200, 1200, 0, flags); /* no write */
       write(fd, "*X", 2);
-      setspeed(fd, 1200, opt_baud, 0, flags); /* no write */
+      setspeed(fd, 1200, (which_mouse->opt_baud), 0, flags); /* no write */
       return type;
    }
 
@@ -1467,11 +1693,22 @@ static Gpm_Type* I_serial(int fd, unsigned short flags,
     }
 
    if (type->fun==M_synaptics_serial) {
+      int packet_length;
+
       setspeed (fd, 1200, 1200, 1, flags);
-      syn_serial_init (fd);
+      packet_length = syn_serial_init (fd);
       setspeed (fd, 1200, 9600, 1, flags);
+
+      type->packetlen = packet_length;
+      type->howmany   = packet_length;
     }
 
+   if (type->fun==M_vsxxx_aa) {
+      setspeed (fd, 4800, 4800, 0, flags); /* no write */
+      write(fd, "R", 1); /* initialize a mouse; without getting an "R" */
+                         /* a mouse does not send a bytestream         */
+   }
+
    return type;
 }
 
@@ -1498,15 +1735,15 @@ static Gpm_Type* I_logi(int fd, unsigned short flags,
    type->howmany = busmouse ? 3 : 1;
 
    /* change from any available speed to the chosen one */
-   for (i=9600; i>=1200; i/=2) setspeed(fd, i, opt_baud, 1 /* write */, flags);
+   for (i=9600; i>=1200; i/=2) setspeed(fd, i, (which_mouse->opt_baud), 1 /* write */, flags);
 
    /* this stuff is peculiar of logitech mice, also for the serial ones */
    write(fd, "S", 1);
-   setspeed(fd, opt_baud, opt_baud, 1 /* write */,
+   setspeed(fd, (which_mouse->opt_baud), (which_mouse->opt_baud), 1 /* write */,
       CS8 |PARENB |PARODD |CREAD |CLOCAL |HUPCL);
 
    /* configure the sample rate */
-   for (i=0;opt_sample<=sampletab[i].sample;i++) ;
+   for (i=0;(which_mouse->opt_sample)<=sampletab[i].sample;i++) ;
    write(fd,sampletab[i].code,1);
    return type;
 }
@@ -1791,7 +2028,7 @@ static Gpm_Type *I_twid(int fd, unsigned short flags,
    * the twiddler is a serial mouse: just drop dtr
    * and run at 2400 (unless specified differently) 
    */
-   if(opt_baud==DEF_BAUD) opt_baud = 2400;
+   if((which_mouse->opt_baud)==DEF_BAUD) (which_mouse->opt_baud) = 2400;
    argv[1] = "dtr"; /* argv[1] is guaranteed to be NULL (this is dirty) */
    return I_serial(fd, flags, type, argc, argv);
 }
@@ -1801,7 +2038,7 @@ static Gpm_Type *I_calus(int fd, unsigned short flags,
 {
    if (check_no_argv(argc, argv)) return NULL;
 
-   if (opt_baud == 1200) opt_baud=9600; /* default to 9600 */
+   if ((which_mouse->opt_baud) == 1200) (which_mouse->opt_baud)=9600; /* default to 9600 */
    return I_serial(fd, flags, type, argc, argv);
 }
 
@@ -1947,10 +2184,10 @@ static Gpm_Type *I_gunze(int fd, unsigned short flags,
    parse_argv(optioninfo, argc, argv);
     
    /* check that the baud rate is valid */
-   if (opt_baud == DEF_BAUD) opt_baud = 19200; /* force 19200 as default */
-   if (opt_baud != 9600 && opt_baud != 19200) {
-      gpm_report(GPM_PR_ERR,GPM_MESS_GUNZE_WRONG_BAUD,prgname, argv[0]);
-      opt_baud = 19200;
+   if ((which_mouse->opt_baud) == DEF_BAUD) (which_mouse->opt_baud) = 19200; /* force 19200 as default */
+   if ((which_mouse->opt_baud) != 9600 && (which_mouse->opt_baud) != 19200) {
+      gpm_report(GPM_PR_ERR,GPM_MESS_GUNZE_WRONG_BAUD,option.progname, argv[0]);
+      (which_mouse->opt_baud) = 19200;
    }
    tcgetattr(fd, &tty);
    tty.c_iflag = IGNBRK | IGNPAR;
@@ -1959,7 +2196,7 @@ static Gpm_Type *I_gunze(int fd, unsigned short flags,
    tty.c_line = 0;
    tty.c_cc[VTIME] = 0;
    tty.c_cc[VMIN] = 1;
-   tty.c_cflag = (opt_baud == 9600 ? B9600 : B19200) |CS8|CREAD|CLOCAL|HUPCL;
+   tty.c_cflag = ((which_mouse->opt_baud) == 9600 ? B9600 : B19200) |CS8|CREAD|CLOCAL|HUPCL;
    tcsetattr(fd, TCSAFLUSH, &tty);
 
    /* FIXME: try to find some information about the device */
@@ -1975,17 +2212,72 @@ static Gpm_Type *I_gunze(int fd, unsigned short flags,
       for (i=0; i<4; i++)
          if (gunze_calib[i] & ~1023) calibok = 0;
       if (gunze_calib[0] == gunze_calib[2]) calibok = 0;
-      if (gunze_calib[1] == gunze_calib[4]) calibok = 0;
+      if (gunze_calib[1] == gunze_calib[3]) calibok = 0;
       fclose(f);
    }
    if (!calibok) {
-      gpm_report(GPM_PR_ERR,GPM_MESS_GUNZE_CALIBRATE, prgname);
+      gpm_report(GPM_PR_ERR,GPM_MESS_GUNZE_CALIBRATE, option.progname);
       gunze_calib[0] = gunze_calib[1] = 128; /* 1/8 */
       gunze_calib[2] = gunze_calib[3] = 896; /* 7/8 */
    }
    return type;
 }
 
+
+/* simple initialization for the elo touchscreen */
+static Gpm_Type *I_etouch(int fd, unsigned short flags,
+			  struct Gpm_Type *type, int argc, char **argv)
+{
+  struct termios tty;
+  FILE *f;
+  char s[80];
+  int i, calibok = 0;
+
+  /* Calibration config file (copied from I_gunze, below :) */
+  #define ELO_CALIBRATION_FILE SYSCONFDIR "/gpm-calibration"
+   /* accept a few options */
+   static argv_helper optioninfo[] = {
+         {"clickontouch",  ARGV_BOOL, u: {iptr: &elo_click_ontouch}, value: !0},
+         {"",       ARGV_END} 
+   };
+   parse_argv(optioninfo, argc, argv);
+
+  /* Set speed to 9600bps (copied from I_summa, above :) */
+  tcgetattr(fd, &tty);
+  tty.c_iflag = IGNBRK | IGNPAR;
+  tty.c_oflag = 0;
+  tty.c_lflag = 0;
+  tty.c_line = 0;
+  tty.c_cc[VTIME] = 0;
+  tty.c_cc[VMIN] = 1;
+  tty.c_cflag = B9600|CS8|CREAD|CLOCAL|HUPCL;
+  tcsetattr(fd, TCSAFLUSH, &tty);
+
+  /* retrieve calibration, if not existent, use defaults (uncalib) */
+  f = fopen(ELO_CALIBRATION_FILE, "r");
+  if (f) {
+	fgets(s, 80, f); /* discard the comment */
+	if (fscanf(f, "%d %d %d %d", gunze_calib, gunze_calib+1,
+		   gunze_calib+2, gunze_calib+3) == 4)
+	    calibok = 1;
+	/* Hmm... check */
+	for (i=0; i<4; i++)
+	    if ((gunze_calib[i] & 0xfff) != gunze_calib[i]) calibok = 0;
+	if (gunze_calib[0] == gunze_calib[2]) calibok = 0;
+	if (gunze_calib[1] == gunze_calib[3]) calibok = 0;
+	fclose(f);
+  }
+  if (!calibok) {
+        gpm_report(GPM_PR_ERR,GPM_MESS_ELO_CALIBRATE, option.progname, ELO_CALIBRATION_FILE);
+        /* "%s: etouch: calibration file %s absent or invalid, using defaults" */
+	gunze_calib[0] = gunze_calib[1] = 0x010; /* 1/16 */
+	gunze_calib[2] = gunze_calib[3] = 0xff0; /* 15/16 */
+  }
+  return type;
+}
+
+
+
 /*  Genius Wizardpad tablet  --  Matt Kimball (mkimball@xmission.com)  */
 static Gpm_Type *I_wp(int fd, unsigned short flags,
             struct Gpm_Type *type, int argc, char **argv)
@@ -2102,13 +2394,16 @@ Gpm_Type mice[]={
    {"calr", "Calcomp UltraSlate - relative mode",
            "", M_calus_rel, I_calus, CS8 | CSTOPB | STD_FLG,
                                 {0x80, 0x80, 0x80, 0x00}, 6, 6, 0, 0, 0},
+  {"etouch",  "EloTouch touch-screens (only button-1 events, by now)",
+           "", M_etouch, I_etouch, STD_FLG,
+                                {0xFF, 0x55, 0xFF, 0x54}, 7, 1, 0, 1, NULL}, 
 #ifdef HAVE_LINUX_INPUT_H
    {"evdev", "Linux Event Device",
             "", M_evdev, I_empty, STD_FLG,
                         {0x00, 0x00, 0x00, 0x00} , 16, 16, 0, 0, NULL},
 #endif /* HAVE_LINUX_INPUT_H */
    {"exps2",   "IntelliMouse Explorer (ps2) - 3 buttons, wheel unused",
-           "ExplorerPS/2", M_ps2, I_exps2, STD_FLG,
+           "ExplorerPS/2", M_imps2, I_exps2, STD_FLG,
                                 {0xc0, 0x00, 0x00, 0x00}, 4, 1, 0, 0, 0},
 #ifdef HAVE_LINUX_JOYSTICK_H
    {"js",   "Joystick mouse emulation",
@@ -2135,7 +2430,7 @@ Gpm_Type mice[]={
                                 {0xe0, 0x80, 0x80, 0x00}, 3, 1, 0, 0, 0},
    {"ms3", "Microsoft Intellimouse (serial) - 3 buttons, wheel unused",
            "", M_ms3, I_pnp, CS7 | STD_FLG,
-                                {0xc0, 0x40, 0xc0, 0x00}, 4, 1, 0, 0, 0},
+                                {0xc0, 0x40, 0xc0, 0x00}, 4, 1, 0, 0, R_ms3},
    {"ms+", "Like 'ms', but allows dragging with the middle button.",
            "", M_ms_plus, I_serial, CS7 | STD_FLG,
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0, 0},
@@ -2162,7 +2457,7 @@ Gpm_Type mice[]={
                                 {0xc0, 0x00, 0x00, 0x00}, 3, 1, 0, 0, R_ps2},
    {"sun",  "'msc' protocol, but only 3 bytes per packet.",
            "", M_sun, I_serial, CS8 | CSTOPB | STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00}, 3, 1, 0, 0, 0},
+                                {0xf8, 0x80, 0x00, 0x00}, 3, 1, 0, 0, R_sun},
    {"summa",  "Summagraphics or Genius tablet absolute mode(MM-Series)",
            "", M_summa, I_summa, STD_FLG,
                                 {0x80, 0x80, 0x00, 0x00}, 5, 1, 0, 1, R_summa},
@@ -2175,6 +2470,9 @@ Gpm_Type mice[]={
    {"twid", "Twidddler keyboard",
            "", M_twid, I_twid, CS8 | STD_FLG,
                                 {0x80, 0x00, 0x80, 0x80}, 5, 1, 0, 0, 0},
+   {"vsxxxaa", "The DEC VSXXX-AA/GA serial mouse on DEC workstations.",
+           "", M_vsxxx_aa, I_serial, CS8 | PARENB | PARODD | STD_FLG,
+                                {0xe0, 0x80, 0x80, 0x00}, 3, 1, 0, 0, 0},
    {"wacom","Wacom Protocol IV Tablets: Pen+Mouse, relative+absolute mode",
            "", M_wacom, I_wacom, STD_FLG,
                                 {0x80, 0x80, 0x80, 0x00}, 7, 1, 0, 0, 0},
@@ -2193,7 +2491,7 @@ int M_listTypes(void)
 {
    Gpm_Type *type;
 
-   printf(GPM_MESS_S_S_N,GPM_NAME,GPM_RELEASE,GPM_DATE);
+   printf(GPM_MESS_VERSION "\n");
    printf(GPM_MESS_AVAIL_MYT);
    for (type=mice; type->fun; type++)
       printf(GPM_MESS_SYNONYM, type->repeat_fun?'*':' ',
