@@ -1,9 +1,10 @@
 /*
  * selects where we should report to
  * TODO: - possibly include errno (%m) string in syslog
- *       - remove unecessary code (if present)
+ *       - njak ... someone should optimize this code and remove double
+ *         fragments.... (self)
  *
- * Copyright (c) 2001        Nico Schottelius <nico@schottelius.org>
+ * Copyright (c) 2001-2008   Nico Schottelius <nico-gpm2008 at schottelius.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -30,13 +31,14 @@
 
 #include "headers/gpmInt.h"
 #include "headers/message.h"
+#include "headers/daemon.h"   /* FIXME: this is wrong; we need report fpr lib und daemon */
 
 /*
  * gpm_report
  *
  * Destinations:
  * - syslog
- * - current console (/dev/vc/0)
+ * - current console (/dev/vc/0 || /dev/tty0)
  * - system console (/dev/console)
  * - stdout
  * - stderr
@@ -65,63 +67,69 @@
  * Client Mode: (to use with mouse-test etc. ; NON SERVER )
  *   debug/err/warn/info: like debugging mode!
  *
- * If the stat is GPM_QUITS, we return the address of the string, else NULL.
- *
  */
 
 void gpm_report(int line, char *file, int stat, char *text, ... )
 {
-   extern int run_status; /* daemon / startup / debug */
-   extern char *consolename;
    FILE *console = NULL;
-   va_list ap;
+   va_list ap, ap3;
+#ifdef HAVE_VSYSLOG
+   va_list ap2;
+#endif
 
    va_start(ap,text);
+   va_copy(ap3, ap);
+#ifdef HAVE_VSYSLOG
+   va_copy(ap2, ap);
+#endif
 
-   switch(run_status) {
+   switch(option.run_status) {
       /******************** STARTUP *****************/
       case GPM_RUN_STARTUP:
          switch(stat) {
             case GPM_STAT_INFO:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_INFO | LOG_USER, GPM_STRING_INFO);
-               vsyslog(LOG_INFO | LOG_USER, text, ap);
-#endif               
-               fprintf(stderr,GPM_STRING_INFO); fprintf(stderr,"\n");
+               vsyslog(LOG_INFO | LOG_USER, text, ap2);
+#else
+               fprintf(stderr,GPM_STRING_INFO);
                vfprintf(stderr,text,ap);
                fprintf(stderr,"\n");
+#endif
                break;
 
             case GPM_STAT_WARN:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_DAEMON | LOG_WARNING, GPM_STRING_WARN);
-               vsyslog(LOG_DAEMON | LOG_WARNING, text, ap);
-#endif               
-               fprintf(stderr,GPM_STRING_WARN); fprintf(stderr,"\n");
+               vsyslog(LOG_DAEMON | LOG_WARNING, text, ap2);
+#else
+               fprintf(stderr,GPM_STRING_WARN);
                vfprintf(stderr,text,ap);
                fprintf(stderr,"\n");
+#endif
                break;
 
             case GPM_STAT_ERR:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_DAEMON | LOG_ERR, GPM_STRING_ERR);
-               vsyslog(LOG_DAEMON | LOG_ERR, text, ap);
-#endif               
-               fprintf(stderr,GPM_STRING_ERR); fprintf(stderr,"\n");
+               vsyslog(LOG_DAEMON | LOG_ERR, text, ap2);
+#else
+               fprintf(stderr,GPM_STRING_ERR);
                vfprintf(stderr,text,ap);
                fprintf(stderr,"\n");
+#endif
                break;
 
             case GPM_STAT_OOPS:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_DAEMON | LOG_ERR, GPM_STRING_OOPS);
-               vsyslog(LOG_DAEMON | LOG_ERR, text, ap);
+               vsyslog(LOG_DAEMON | LOG_ERR, text, ap2);
 #endif               
-               fprintf(stderr,GPM_STRING_OOPS); fprintf(stderr,"\n");
+               fprintf(stderr,GPM_STRING_OOPS);
                vfprintf(stderr,text,ap);
                fprintf(stderr,"\n");
 
-               exit(1); /*we should have a oops()function, but this works, too*/
+               exit(1); /* we should have a oops()-function,but this works,too*/
                break;
          }
          break; /* startup sequence */   
@@ -132,49 +140,51 @@ void gpm_report(int line, char *file, int stat, char *text, ... )
             case GPM_STAT_INFO:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_INFO | LOG_USER, GPM_STRING_INFO);
-               vsyslog(LOG_INFO | LOG_USER, text, ap);
+               vsyslog(LOG_INFO | LOG_USER, text, ap2);
 #endif
                break;
 
             case GPM_STAT_WARN:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_DAEMON | LOG_WARNING, GPM_STRING_WARN);
-               vsyslog(LOG_DAEMON | LOG_WARNING, text, ap);
-#endif               
+               vsyslog(LOG_DAEMON | LOG_WARNING, text, ap2);
+#else
                if((console = fopen(GPM_SYS_CONSOLE,"a")) != NULL) {
-                  fprintf(console,GPM_STRING_WARN); fprintf(console,"\n");
+                  fprintf(console,GPM_STRING_WARN);
                   vfprintf(console,text,ap);
                   fprintf(console,"\n");
                   fclose(console);
                }   
+#endif
                break;
  
             case GPM_STAT_ERR:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_DAEMON | LOG_ERR, GPM_STRING_ERR);
-               vsyslog(LOG_DAEMON | LOG_ERR, text, ap);
-#endif               
+               vsyslog(LOG_DAEMON | LOG_ERR, text, ap2);
+#else
                if((console = fopen(GPM_SYS_CONSOLE,"a")) != NULL) {
-                  fprintf(console,GPM_STRING_ERR); fprintf(console,"\n");
+                  fprintf(console,GPM_STRING_ERR);
                   vfprintf(console,text,ap);
                   fprintf(console,"\n");
                   fclose(console);
                }
 
-               if((console = fopen(consolename,"a")) != NULL) {
-                  fprintf(console,GPM_STRING_ERR); fprintf(console,"\n");
-                  vfprintf(console,text,ap);
+               if((console = fopen(option.consolename,"a")) != NULL) {
+                  fprintf(console,GPM_STRING_ERR);
+                  vfprintf(console,text,ap3);
                   fprintf(console,"\n");
                   fclose(console);
                }
+#endif
                break;
 
             case GPM_STAT_OOPS:
 #ifdef HAVE_VSYSLOG
                syslog(LOG_DAEMON | LOG_ERR, GPM_STRING_OOPS);
-               vsyslog(LOG_DAEMON | LOG_ERR, text, ap);
+               vsyslog(LOG_DAEMON | LOG_ERR, text, ap2);
 #endif               
-               fprintf(stderr,GPM_STRING_OOPS); fprintf(stderr,"\n");
+               fprintf(stderr,GPM_STRING_OOPS);
                vfprintf(stderr,text,ap);
                fprintf(stderr,"\n");
 
@@ -203,7 +213,6 @@ void gpm_report(int line, char *file, int stat, char *text, ... )
                fprintf(console,GPM_STRING_OOPS); break;
          }
 
-         fprintf(console,"\n"); 
          vfprintf(console,text,ap);
          fprintf(console,"\n");
          
@@ -211,6 +220,12 @@ void gpm_report(int line, char *file, int stat, char *text, ... )
 
          break;
    } /* switch for current modus */
+
+   va_end(ap);
+   va_end(ap3);
+#ifdef HAVE_VSYSLOG
+   va_end(ap2);
+#endif
 } /* gpm_report */   
 
 
